@@ -118,7 +118,7 @@ static struct timeval shout_connection_iter__wait_for_io__get_timeout(shout_conn
             .tv_usec = (timeout % 1000) * 1000
         };
         return tv;
-    } else if (con->nonblocking) {
+    } else if (con->nonblocking == SHOUT_BLOCKING_NONE) {
         return tv_nonblocking;
     } else {
         return tv_blocking;
@@ -167,7 +167,7 @@ static shout_connection_return_state_t shout_connection_iter__socket(shout_conne
             }
         break;
         case SHOUT_SOCKSTATE_CONNECTING:
-            if (con->nonblocking) {
+            if (con->nonblocking == SHOUT_BLOCKING_NONE) {
                 ret = shout_connection_iter__wait_for_io(con, shout, 1, 1, 0);
                 if (ret != SHOUT_RS_DONE) {
                     return ret;
@@ -460,7 +460,7 @@ int                 shout_connection_iter(shout_connection_t *con, shout_t *shou
             break; \
             case SHOUT_RS_TIMEOUT: \
             case SHOUT_RS_NOTNOW: \
-                if (con->nonblocking) \
+                if (con->nonblocking == SHOUT_BLOCKING_NONE) \
                     return SHOUTERR_RETRY; \
                 retry = 1; \
             break; \
@@ -518,7 +518,7 @@ int                 shout_connection_select_tlsmode(shout_connection_t *con, int
 }
 int                 shout_connection_set_nonblocking(shout_connection_t *con, unsigned int nonblocking)
 {
-    if (!con)
+    if (!con || (nonblocking != SHOUT_BLOCKING_DEFAULT && nonblocking != SHOUT_BLOCKING_FULL && nonblocking != SHOUT_BLOCKING_NONE))
         return SHOUTERR_INSANE;
 
     if (con->socket != SOCK_ERROR)
@@ -563,13 +563,14 @@ int                 shout_connection_connect(shout_connection_t *con, shout_t *s
     if (con->socket != SOCK_ERROR || con->current_socket_state != SHOUT_SOCKSTATE_UNCONNECTED)
         return SHOUTERR_BUSY;
 
-    shout_connection_set_nonblocking(con, shout_get_nonblocking(shout));
+    if (con->nonblocking == SHOUT_BLOCKING_DEFAULT)
+        shout_connection_set_nonblocking(con, shout_get_nonblocking(shout));
 
     port = shout->port;
-    if (shout_get_protocol(shout) == SHOUT_PROTOCOL_ICY)
+    if (con->impl == shout_icy_impl)
         port++;
 
-    if (con->nonblocking) {
+    if (con->nonblocking == SHOUT_BLOCKING_NONE) {
         con->socket = sock_connect_non_blocking(shout->host, port);
     } else {
         con->socket = sock_connect(shout->host, port);
@@ -620,6 +621,9 @@ ssize_t             shout_connection_send(shout_connection_t *con, shout_t *shou
     if (con->current_message_state != SHOUT_MSGSTATE_SENDING1)
         return -1;
 
+    if (con->error == SHOUTERR_SOCKET)
+        return -1;
+
     ret = shout_queue_data(&(con->wqueue), buf, len);
     if (ret != SHOUTERR_SUCCESS) {
         shout_connection_set_error(con, ret);
